import sys
import importlib
import copy
from io import StringIO
from contextlib import redirect_stdout

import pytest

import installed


def test_polog_install_and_import():
    with installed('polog'):
        import polog
        importlib.reload(polog)


def test_polog_install_and_import_with_logger_none():
    with installed('polog', logger=None):
        import polog
        importlib.reload(polog)


def test_polog_install_two_and_import():
    with installed('polog', 'astrologic'):
        import polog
        import astrologic
        importlib.reload(polog)
        importlib.reload(astrologic)


def test_polog_install_two_contexts_and_import():
    with installed('polog'):
        with installed('astrologic'):
            import polog
            import astrologic
            importlib.reload(polog)
            importlib.reload(astrologic)


def test_deleting_contexts():
    with installed('polog'):
        with installed('astrologic'):
            pass
    with pytest.raises(ModuleNotFoundError):
        import polog
        importlib.reload(polog)


def test_sys_path_lenth():
    number_before = len(sys.path)
    sys_path_copy = copy.copy(sys.path)

    with installed('polog'):
        assert len(sys.path) == number_before + 1
        assert sys.path[1:] == sys_path_copy

    assert len(sys.path) == number_before


def test_fazy_install_and_autoimport():
    with installed('fazy==0.0.2') as package:
        f = package.import_here('f')
        assert f('kek') == 'kek'


def test_super_project():
    with installed('super_test_project', index_url='https://test.pypi.org/simple/') as package:
        module = package.import_here('super_test')
        assert module.version == '0.0.2'
        assert module.function(2, 3) == 5

    with installed('super_test_project==0.0.1', index_url='https://test.pypi.org/simple/') as package:
        module = package.import_here('super_test')
        assert module.version == '0.0.1'
        assert module.function(2, 3) == 5

    with installed('super_test_project==0.0.2', index_url='https://test.pypi.org/simple/') as package:
        module = package.import_here('super_test')
        assert module.version == '0.0.2'
        assert module.function(2, 3) == 5

    with installed('super_test_project==0.0.1', index_url='https://test.pypi.org/simple/') as package:
        module = package.import_here('super_test')
        assert module.version == '0.0.1'
        assert module.function(2, 3) == 5


def test_super_project_nested():
    with installed('super_test_project==0.0.1', index_url='https://test.pypi.org/simple/') as package_1:
        with installed('super_test_project==0.0.2', index_url='https://test.pypi.org/simple/') as package_2:
            with installed('super_test_project==0.0.1', index_url='https://test.pypi.org/simple/') as package_3:
                module_1 = package_1.import_here('super_test')
                module_2 = package_2.import_here('super_test')
                module_3 = package_3.import_here('super_test')
                assert module_1.version == '0.0.1'
                assert module_2.version == '0.0.2'
                assert module_3.version == '0.0.1'



def test_catch_output_default():
    with redirect_stdout(StringIO()) as context:
        with installed('super_test_project==0.0.1', index_url='https://test.pypi.org/simple/'):
            pass

    assert len(context.getvalue()) > 0


def test_catch_output_false():
    with redirect_stdout(StringIO()) as context:
        with installed('super_test_project==0.0.1', index_url='https://test.pypi.org/simple/', catch_output=False):
            pass

    assert len(context.getvalue()) > 0


def test_catch_output_true():
    with redirect_stdout(StringIO()) as context:
        with installed('super_test_project==0.0.1', index_url='https://test.pypi.org/simple/', catch_output=True):
            pass

    assert len(context.getvalue()) == 0
